ScriptName easywheelapi extends Form hidden
{Exposed API.}


Function RaiseEWMAPIError(String ErrorMessage, Int severity) GLOBAL
	debug.Trace("[EasyWheel] Error : " + ErrorMessage, 0)
EndFunction

Function SendEWMEvent(String functionName) GLOBAL
	Int handle = modevent.Create("EWM_" + functionName)
	IF handle
		modevent.send(handle)
	ENDIF
EndFunction

formlist Function GetHandlers() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	RETURN EWMAPI.EWM_HANDLER_LIST
EndFunction

Int Function GetFunctionsCount() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN 0
	ENDIF
	RETURN EWMAPI._GetFunctionsCount()
EndFunction

Function AddFunctions(String[] functions) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
	ENDIF
	EWMAPI._AddFunctions(functions)
EndFunction

Function TryOpenWheel(String[] akLayout) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
	ENDIF
	ui.CloseCustomMenu()
	EWMAPI.CurrentLayoutIndex = easywheelapi.GetLayoutIndex(akLayout)
	IF !(EWMAPI.WheelMenu.Open(akLayout))
		easywheelapi.RaiseEWMAPIError(("Layout " + (EWMAPI.CurrentLayoutIndex)) + " corrupted, cleaning now", 0)
		easywheelapi.AutofillLayout(akLayout, true)
	ENDIF
EndFunction

Int Function GetHandlersCount() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN 0
	ENDIF
	RETURN EWMAPI.EWM_HANDLER_LIST.GetSize()
EndFunction

ewm_handlemodbase Function GetHandlerWithFunction(String functionName) GLOBAL
	Int i = easywheelapi.GetHandlersCount()
	WHILE i > 0
		i = i - 1
		IF (easywheelapi.GetHandler(i).HasFunction(functionName)) >= 0
			RETURN easywheelapi.GetHandler(i)
		ENDIF
	ENDWHILE
EndFunction

Function AutofillLayout(String[] wheelLayout, Bool blankOnly) GLOBAL
	Int i = -1
	IF !blankOnly
		Int n = easywheelapi.GetFunctionsCount()
		WHILE i < n && i < 7
			i = i + 1
			wheelLayout[i] = easywheelapi.GetFunction(i)
			IF (wheelLayout[i]) == ""
			ENDIF
		ENDWHILE
	ELSE
		wheelLayout = new String[8]
	ENDIF
	WHILE i < 7
		i = i + 1
		wheelLayout[i] = "EMPTY"
	ENDWHILE
EndFunction

Function WheelOpen(Bool isOpen) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	EWMAPI._isWheelOpen = isOpen
EndFunction

Bool Function bHideSpells() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN false
	ENDIF
	RETURN EWMAPI._bHideSpells
EndFunction

Function RemoveFunctions(String[] functions) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
	ENDIF
EndFunction

Function ResetKeycodes() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
	ENDIF
	EWMAPI._ResetKeycodes()
EndFunction

Function RefreshFunctions() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	EWMAPI.EWM_FCT_ALL = new String[128]
	Int n = EWMAPI.EWM_HANDLER_LIST.GetSize()
	WHILE n > 0
		n = n - 1
		easywheelapi.GetHandler(n).RegisterFunctions()
	ENDWHILE
EndFunction

Int Function GetCurrentIndex() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN 0
	ENDIF
	RETURN EWMAPI.CurrentLayoutIndex
EndFunction

Function SetKeycodeWheel(Int wheelNumber, Int keyCode) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	IF !(EWMAPI.WHEELS_KEYCODES)
		easywheelapi.RaiseEWMAPIError("Wheels keycodes not initialized", 0)
		easywheelapi.ResetKeycodes()
		RETURN NONE
	ENDIF
	IF wheelNumber < 0 || wheelNumber > 15
		RETURN NONE
	ENDIF
	EWMAPI.WHEELS_KEYCODES[wheelNumber - 1] = keyCode
EndFunction

Function SetInputState(String stateName) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
	ENDIF
	EWMAPI.InputState = stateName
EndFunction

Int Function GetKeycodeWheel(Int wheelNumber) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN 0
	ENDIF
	IF !(EWMAPI.WHEELS_KEYCODES)
		easywheelapi.RaiseEWMAPIError("Wheels keycodes not initialized", 0)
		easywheelapi.ResetKeycodes()
		RETURN 0
	ENDIF
	IF wheelNumber < 1 || wheelNumber > 16
		RETURN 0
	ENDIF
	RETURN EWMAPI.WHEELS_KEYCODES[wheelNumber - 1]
EndFunction

Bool Function ChangeShowHandlerStatus() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN false
	ENDIF
	EWMAPI._bShowHandlerStatus = !(EWMAPI._bShowHandlerStatus)
	RETURN EWMAPI._bShowHandlerStatus
EndFunction

Bool Function GetFunctionCondition(String functionName) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	Int i = EWMAPI.EWM_HANDLER_LIST.GetSize()
	WHILE i > 0
		i = i - 1
		ewm_handlemodbase currentHandler = (EWMAPI.EWM_HANDLER_LIST.GetAt(i)) as ewm_handlemodbase
		Int functionIndexInHandler = currentHandler.HasFunction(functionName)
		IF functionIndexInHandler >= 0
			RETURN currentHandler.GetFunctionCondition(functionIndexInHandler)
		ENDIF
	ENDWHILE
EndFunction

Bool Function ChangeShowPlayerStats() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN false
	ENDIF
	EWMAPI._bShowPlayerStats = !(EWMAPI._bShowPlayerStats)
	RETURN EWMAPI._bShowPlayerStats
EndFunction

Function SetKeycodeSwap(String option, Int keyCode) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	IF option == "Show"
		EWMAPI.SWAP_WHEELS_SHOW_KEYCODE = keyCode
	ELSEIF option == "Next"
		EWMAPI.SWAP_WHEELS_NEXT_KEYCODE = keyCode
	ELSEIF option == "Prev"
		EWMAPI.SWAP_WHEELS_PREV_KEYCODE = keyCode
	ENDIF
EndFunction

Bool Function IsEmptyLayout(String[] akLayout) GLOBAL
	Int i = akLayout.length
	IF !(i == 8)
		easywheelapi.RaiseEWMAPIError("Layout doesn't have 8 entries", 0)
		RETURN true
	ENDIF
	WHILE i > 0
		i = i - 1
		IF !((akLayout[i]) == "") && !((akLayout[i]) == "EMPTY")
			RETURN false
		ENDIF
	ENDWHILE
	RETURN true
EndFunction

Bool Function ChangeSpellsVisibility() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN false
	ENDIF
	EWMAPI._bHideSpells = !(EWMAPI._bHideSpells)
	EWMAPI._ShowSpells()
	IF EWMAPI._bHideSpells
		EWMAPI._HideSpells()
	ENDIF
	RETURN EWMAPI._bHideSpells
EndFunction

_EasyWheelAPI Function GetAPI() GLOBAL
	RETURN ((game.GetFormFromFile(3426, "EasyWheel-AIO.esp")) as quest) as _EasyWheelAPI
EndFunction

Function EnableHandler(String QuestHandlerName, String akPluginName) GLOBAL
	ewm_handlemodbase handler = Quest.GetQuest(QuestHandlerName) as ewm_handlemodbase
	If handler
		IF !(akPluginName == "")
			handler.PluginName = akPluginName
		ENDIF
		IF !((handler.GetState()) == "HandlerON")
			handler.GotoState("HandlerON")
		ENDIF
	Else
		Debug.Trace("Quest " + QuestHandlerName + " not found.")
	EndIf
EndFunction

String Function GetFunction(Int index) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	RETURN EWMAPI.EWM_FCT_ALL[index]
EndFunction

Function ResetConfig() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	easywheelapi.ResetKeycodes()
	Int i = 2
	WHILE i <= 16
		easywheelapi.CleanLayout(i)
		i = i + 1
	ENDWHILE
	easywheelapi.AutofillLayout(easywheelapi.GetLayout(1), false)
	easywheelapi.SetInputState("WheelPerKey")
	IF EWMAPI._bHideSpells
		EWMAPI._HideSpells()
	ENDIF
EndFunction

Function CheckMods() GLOBAL
	IF !(ewm_modscanner.ScanInstalledMods())
		easywheelapi.EnableWheelMenuAPI(false)
		easywheelapi.RaiseEWMAPIError("Mod Scanner Failed.", 0)
		debug.MessageBox("EasyWheel : \n Pre-requisites missing, mod disabled.")
		RETURN NONE
	ENDIF
	easywheelapi.EnableWheelMenuAPI(true)
EndFunction

Function RefreshKeyCodes() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	EWMAPI.InputManager.RefreshKeys()
EndFunction

String[] Function GetPrevLayout() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	Int prevIndex = (EWMAPI.CurrentLayoutIndex) - 1
	IF prevIndex == 0
		prevIndex = 16
	ENDIF
	Int counter = 0
	WHILE easywheelapi.IsEmptyLayout(easywheelapi.GetLayout(prevIndex)) || counter > 16
		counter = counter + 1
		prevIndex = prevIndex - 1
		IF prevIndex == 0
			prevIndex = 16
		ENDIF
	ENDWHILE
	IF counter > 16
		easywheelapi.RaiseEWMAPIError("All layouts are empty", 0)
	ENDIF
	RETURN easywheelapi.GetLayout(prevIndex)
EndFunction

String[] Function GetNextLayout() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	Int nextIndex = (EWMAPI.CurrentLayoutIndex) + 1
	IF nextIndex == 17
		nextIndex = 1
	ENDIF
	Int counter = 0
	WHILE easywheelapi.IsEmptyLayout(easywheelapi.GetLayout(nextIndex)) || counter > 16
		counter = counter + 1
		nextIndex = nextIndex + 1
		IF nextIndex == 17
			nextIndex = 1
		ENDIF
	ENDWHILE
	IF counter > 16
		easywheelapi.RaiseEWMAPIError("All layouts are empty", 0)
	ENDIF
	RETURN easywheelapi.GetLayout(nextIndex)
EndFunction

Int Function GetLayoutIndex(String[] akLayout) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN 0
	ENDIF
	RETURN EWMAPI._GetLayoutIndex(akLayout)
EndFunction

String Function GetInputState() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN ""
	ENDIF
	RETURN EWMAPI.InputState
EndFunction

String[] Function LayoutFromKeycode(Int keyCode) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	Int keycodeIndex = EWMAPI.WHEELS_KEYCODES.find(keyCode, 0)
	IF keycodeIndex < 0
		easywheelapi.RaiseEWMAPIError("keyCode doesn't match any known wheel layout", 0)
		RETURN NONE
	ENDIF
	RETURN easywheelapi.GetLayout(keycodeIndex + 1)
EndFunction

Function CleanLayout(Int i) GLOBAL
	easywheelapi.AutofillLayout(easywheelapi.GetLayout(i), true)
EndFunction

Bool Function isWheel(Int wheelNumber) GLOBAL
	IF !((easywheelapi.GetKeycodeWheel(wheelNumber)) == 0)
		RETURN true
	ENDIF
	RETURN false
EndFunction

String[] Function GetCurrentLayout() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	RETURN easywheelapi.GetLayout(EWMAPI.CurrentLayoutIndex)
EndFunction

Function DisableHandler(String QuestHandlerName) GLOBAL
	EWM_handlemodbase handler = Quest.GetQuest(QuestHandlerName) as EWM_handlemodbase
	If handler != none
		handler.GotoState("HandlerOFF")
	EndIf
EndFunction

String[] Function GetVisibleFunctions() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	RETURN EWMAPI._GetVisibleFunctions()
EndFunction

String[] Function GetLayout(Int i) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	RETURN EWMAPI._GetLayout(i)
EndFunction

Int Function KeycodeSwap(String option) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN 0
	ENDIF
	IF option == "" || option == "Show"
		RETURN EWMAPI.SWAP_WHEELS_SHOW_KEYCODE
	ELSEIF option == "Next"
		RETURN EWMAPI.SWAP_WHEELS_NEXT_KEYCODE
	ELSEIF option == "Prev"
		RETURN EWMAPI.SWAP_WHEELS_PREV_KEYCODE
	ELSE
		RETURN 0
	ENDIF
EndFunction

String[] Function GetUserSelection() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	RETURN EWMAPI.EWM_USER_SELECTION
EndFunction

Function EnableWheelMenuAPI(Bool isEnabled) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("Serious shit happened", 0)
	ENDIF
	Int timer = 0
	WHILE !(EWMAPI._isInitialized) && timer <= 10
		utility.Wait(0.200000)
		timer = timer + 1
	ENDWHILE
	IF timer >= 10
		easywheelapi.RaiseEWMAPIError("Easywheel couldn't initialize itself within the 20s limit", 0)
		RETURN NONE
	ENDIF
	EWMAPI._isAvailable = isEnabled
EndFunction

Function RemoveFunction(String aFunction) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
	ENDIF
	easywheelapi.RemoveFunction(aFunction)
EndFunction

ewm_handlemodbase Function GetHandler(Int index) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	RETURN ((EWMAPI.EWM_HANDLER_LIST.GetAt(index)) as quest) as ewm_handlemodbase
EndFunction

Function SetCurrentIndex(Int index) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	EWMAPI.CurrentLayoutIndex = index
EndFunction

String[] Function GetWheels() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	RETURN EWMAPI.wheelList
EndFunction

Bool Function isWheelOpen() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN false
	ENDIF
	RETURN EWMAPI._isWheelOpen
EndFunction

Function AddFunction(String aFunction) GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
	ENDIF
	EWMAPI._AddFunction(aFunction)
EndFunction

String[] Function GetFunctions() GLOBAL
	_EasyWheelAPI EWMAPI = easywheelapi.GetAPI()
	IF EWMAPI == NONE
		easywheelapi.RaiseEWMAPIError("EWMAPI unavailable", 0)
		RETURN NONE
	ENDIF
	RETURN EWMAPI.EWM_FCT_ALL
EndFunction
